/*
 * MathPlanner 3.1 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 * 
 * This program is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 * 
 * You should have received a copy of the GNU General Public License with this program. 
 * 
 */


#include <stdio.h>
#include "AppControl.moc"
#include "Error.h"
#include "Par.h"
#include "Header.h"
#include "Function_object.h"
#include "Object_string.h"
#include "Math.h"
#include "ConfigReader.h"
#include "ServerHeaders.h"
#include "Setup.h"
#include "ListOfGreekAlphabets.h"
#include "Extern.h"
#include "Paper.h"
#include "Datastore.h"


class draw_par_class *draw_par;
class draw_sigma_class *draw_sigma;


void ApplicationControl::CreateControlMenu(m_base_object *obj)
{
  Bug("################ Control Menu Called");

  int ot=0;
  if (obj) ot=obj->Type;
		
	base_object_message *b=base_om[ot];
	
	if (b==NULL && obj) {
			b=base_om[ot]=obj->CreateControlObject();
			if (b) wstack->addWidget(b->MainWidget(),ot);		
	}
	
	if (b) b->ReadState(obj);
  else ot=0; // Call Null Widget

	wstack->raiseWidget(ot);
	wstack->setMaximumWidth(10);
	wstack->setMinimumWidth(wstack->visibleWidget()->minimumWidth());
		
	Bug("###### Control Menu Ends");
}




void ApplicationControl::MessageToActiveObject(int a,int b,QString c)
{
	class m_function_object *func=GetActiveFunction();
	if (func) {
    func->SetModify();
		string_object *str=func->GetFocus();
		if (str) {
			m_base_object *obj=str->GetFocus();
			if (obj) obj->MessageReceived(a,b,c);
		}
	 func->DrawRQ();	
	}	
}




void ApplicationControl::CreateFunctionMenu(QStringList list)
{
	if (function_server) {	
	FunctionMenu->clear();	
	for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
	      FunctionMenu->insertItem(*it);
	}
  }
}




ApplicationControl::ApplicationControl(MathWindow *p)
:QObject(p)
{
	int i;

  DocumentChanged=true;

	for(i=0;i<50;i++) base_om[i]=NULL;
  zoomfactor=100;
	parent=p;

	Prefs=new config_file_reader();
  create_table_of_numbers(500);
  integer_server=new int_server();
	function_server=new function_serv();
  Setup=new setup(parent,Prefs);

  Unit=new mpl_unit(Prefs);

  Bug("Init Draw Par");
  draw_par=new draw_par_class();
  draw_par->Init(0,QString("/Par.xpm"),Prefs->Color("PaperColor"),Prefs->Color("OperatorFontColor"),Prefs->Int("Par_size_factor"));
  draw_par->Init(1,QString("/ParB.xpm"),Prefs->Color("PaperColor"),Prefs->Color("OperatorFontColor"),Prefs->Int("Par_size_factor"));
  draw_par->Init(2,QString("/ParC.xpm"),Prefs->Color("PaperColor"),Prefs->Color("OperatorFontColor"),Prefs->Int("Par_size_factor"));
  draw_par->Init(3,QString("/ParD.xpm"),Prefs->Color("PaperColor"),Prefs->Color("OperatorFontColor"),Prefs->Int("Par_size_factor"));

  draw_sigma=new draw_sigma_class();
  draw_sigma->Init(0,QString("/Sigma.xpm"),Prefs->Color("PaperColor"),Prefs->Color("OperatorFontColor"),Prefs->Int("Sigma_size_factor"));
  draw_sigma->Init(1,QString("/Pii.xpm"),Prefs->Color("PaperColor"),Prefs->Color("OperatorFontColor"),Prefs->Int("Sigma_size_factor"));


	if (!DragImage.load(image_path+QString("/Drag.xpm"))) ErrorReport("BuildMenu","No Image Drag.xpm");
	

	FunctionMenu=new QPopupMenu();
	connect(FunctionMenu,SIGNAL(activated(int)),this,SLOT(SetFunction(int)));
	connect(&Timer,SIGNAL(timeout()),this,SLOT(TimeOut()));
	
	QAccel *a = new QAccel(p);
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelPow"))),this,SLOT(TPower()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelSqrt"))),this,SLOT(TSqrt()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelPar"))),this,SLOT(TParA()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelE"))),this,SLOT(Te()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelExp"))),this,SLOT(TExp()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelLn"))),this,SLOT(Tln()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelLog"))),this,SLOT(TLog()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelSin"))),this,SLOT(TSin()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelCos"))),this,SLOT(TCos()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelTan"))),this,SLOT(TTan()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelAsin"))),this,SLOT(TaSin()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelAcos"))),this,SLOT(TaCos()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelAtan"))),this,SLOT(TaTan()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelImag"))),this,SLOT(TImaginary()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelCang"))),this,SLOT(TCAng()));
  a->connectItem( a->insertItem(QKeySequence(Prefs->String("AccelPiiConstant"))),this,SLOT(TPiiConst()));

  if (Prefs->Int("MapItoImag"))
  a->connectItem( a->insertItem(QKeySequence(Qt::Key_I)),this,SLOT(TImaginary()));
	
 	ControlBar=new QToolBar(parent);
	ObjectBar=new QToolBar(parent);
	
	wstack=new QWidgetStack(ObjectBar);
	wstack->setFixedHeight(24);
  wstack->addWidget(new QWidget(NULL),0);  // Add Null Widget

	OperatorBar=new QToolBar(parent);
	OperatorBar->setOrientation(Qt::Vertical);
	parent->moveToolBar(OperatorBar,QMainWindow::Left);

  ComplexBar=new QToolBar(parent);
	ComplexBar->setOrientation(Qt::Vertical);
	parent->moveToolBar(ComplexBar,QMainWindow::Left);

  if (Prefs->Int("UseGUIfont")) {
    QFont font(Prefs->String("GUIFont"),Prefs->Int("GUIFontSize"));
    OperatorBar->setFont(font);
    ComplexBar->setFont(font);
  }

  Bug("Setting Fonts");


  Clipboard=new DataStorage();
  //Graph=NULL; //new GraphDialog(parent,this);
  SetupFonts();
  ControlBarSetup();
  CreateOperatorBar();
  CreateComplexBar();
}





void ApplicationControl::SetupFonts()
{

QFont::insertSubstitution(Prefs->String("BaseFont"),Prefs->String("SymbolFont"));


font[FONT_TYPE_NORMAL]=QFont(Prefs->String("BaseFont"),12,Prefs->Int("BaseFontWeight"),(bool)Prefs->Int("BaseFontItalic"));
font[FONT_TYPE_OPERATOR]=QFont(Prefs->String("OperatorFont"),12,Prefs->Int("OperatorFontWeight"),(bool)Prefs->Int("OperatorFontItalic"));
font[FONT_TYPE_INDEX]=QFont(Prefs->String("IndexFont"),12,Prefs->Int("IndexFontWeight"),(bool)Prefs->Int("IndexFontItalic"));
font[FONT_TYPE_FUNCTION]=QFont(Prefs->String("FunctionFont"),12,Prefs->Int("FunctionFontWeight"),(bool)Prefs->Int("FunctionFontItalic"));


bool fonterror=false;

QFontInfo info(font[FONT_TYPE_NORMAL]);
if (info.family()!=font[FONT_TYPE_NORMAL].family()) fonterror=true;
QFontInfo info2(font[FONT_TYPE_OPERATOR]);
if (info2.family()!=font[FONT_TYPE_OPERATOR].family()) fonterror=true;
QFontInfo info3(font[FONT_TYPE_INDEX]);
if (info3.family()!=font[FONT_TYPE_INDEX].family()) fonterror=true;
QFontInfo info4(font[FONT_TYPE_FUNCTION]);
if (info4.family()!=font[FONT_TYPE_FUNCTION].family()) fonterror=true;

if (fonterror) {
    QMessageBox::critical( 0, "Font Setup",
	  QObject::tr("Requested font not found. Edit the preferences\n")+
	  QObject::tr("\n\nApplication will now continue." ));
}	



color[FONT_TYPE_NORMAL]=Prefs->Color("BaseFontColor");
color[FONT_TYPE_SYMBOL]=Prefs->Color("SymbolFontColor");
color[FONT_TYPE_OPERATOR]=Prefs->Color("OperatorFontColor");
color[FONT_TYPE_INDEX]=Prefs->Color("IndexFontColor");
color[FONT_TYPE_FUNCTION]=Prefs->Color("FunctionFontColor");

centerline=Prefs->Int("BaseFontCenterLine");

fontsize[FONT_SIZE_NORMAL]=Prefs->Int("FontSizeNormal");
fontsize[FONT_SIZE_SMALL]=Prefs->Int("FontSizeSmall");
fontsize[FONT_SIZE_INDEX]=Prefs->Int("FontSizeIndex");
fontsize[FONT_SIZE_INDEX_SMALL]=Prefs->Int("FontSizeIndexSmall");

fontsize[FONT_SIZE_EXPONENT_SMALL]=Prefs->Int("FontSizeExpSmall");


#ifdef BUG_REPORT

  QFontDatabase db;
  QStringList list2;
  QStringList::Iterator it2;
  QStringList list=db.families();
  for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it ) {
            Bug( (char *)(*it).latin1() );

  }

#endif


}



int ApplicationControl::CenterLine(string_object *str)
{
QFont f=str->GetFont(FONT_TYPE_NORMAL);
QFontMetrics met(f);
int he=met.ascent()*centerline/100;
return(he);
}




void ApplicationControl::ControlBarSetup()
{

	expmodecombo=new QComboBox(ControlBar);
	expmodecombo->setMaximumWidth(120);
	
	expmodecombo->insertItem(QObject::tr("Automatic"));
	expmodecombo->insertItem(QObject::tr("Exponent"));
	expmodecombo->insertItem(QObject::tr("Exp - 3"));
	
	connect(expmodecombo,SIGNAL(activated(int)),this,SLOT(Send(int)));

	digitcount=new QComboBox(ControlBar);
	digitcount->setMaximumWidth(60);
	connect(digitcount,SIGNAL(activated(int)),this,SLOT(Send(int)));
	
	QString str;  int i;
	for(i=2;i<17;i+=2) digitcount->insertItem(str.setNum(i));
	
	ControlBar->addSeparator();

	fontsizecombo=new QComboBox(ControlBar);
	fontsizecombo->setMaximumWidth(80);
	fontsizecombo->insertItem(QObject::tr("Normal"));
	fontsizecombo->insertItem(QObject::tr("Medium"));
  fontsizecombo->insertItem(QObject::tr("Small"));
  connect(fontsizecombo,SIGNAL(activated(int)),this,SLOT(SetFont(int)));


  answercombo=new QComboBox(ControlBar);
	answercombo->setMaximumWidth(160);
	
  answercombo->insertItem(QObject::tr("Component"));
  answercombo->insertItem(QObject::tr("Complex-Deg"));	
  answercombo->insertItem(QObject::tr("Complex-Rad"));

  connect(answercombo,SIGNAL(activated(int)),this,SLOT(Send(int)));

  ControlBar->addSeparator();
	rational=new QPushButton(ControlBar);
  rational->setMaximumWidth(40);
  rational->setMaximumHeight(18);
  rational->setToggleButton(true);
  rational->setText("Rat");
  connect(rational,SIGNAL(clicked()),this,SLOT(Send()));
	
	ControlBar->addSeparator();
	
	zoomcombo=new QComboBox(ControlBar);
	zoomcombo->setMaximumWidth(60);
	zoomcombo->insertItem(QString("50"));
	zoomcombo->insertItem(QString("75"));
	zoomcombo->insertItem(QString("100"));
	zoomcombo->insertItem(QString("150"));
	zoomcombo->insertItem(QString("200"));
	connect(zoomcombo,SIGNAL(activated(int)),this,SLOT(ZoomChanged(int)));
	
	QToolTip::add(expmodecombo,tr("Select exponent mode"));
	QToolTip::add(fontsizecombo,tr("Select font size"));
  QToolTip::add(rational,tr("Show the answer in rational mode if possible"));
	QToolTip::add(digitcount,tr("Set digitcount"));	
  QToolTip::add(zoomcombo,tr("Select zoom factor"));
  QToolTip::add(answercombo,tr("Set display style of answer (complex only)"));			
}



void ApplicationControl::SetFunction(int id)
{

SelectedFunction=FunctionMenu->text(id);
Command(T_FUNCTION);

}



void ApplicationControl::TimeOut()
{
  Paper *pap=GetActivePaper();
  if (pap) pap->StartDragging();
}



void ApplicationControl::Send(int)
{
  Send();
}



void ApplicationControl::Send()
{
  SendFunctionStatus();
  m_function_object *func=GetActiveFunction();
  if (func) func->DrawRQ();
}



void ApplicationControl::SetFont(int x)
{
  class m_function_object *func=GetActiveFunction();
  class string_object *string;
	if (func) {
      func->SetModify();	
      string=func->GetFocus();
      if (string) string->SetFontSize(x);
	}	
}



void ApplicationControl::Read()
{
  RecFunctionStatus();
}	




m_function_object *ApplicationControl::GetActiveFunction()
{
  class Paper *pap;
  if (parent) pap=parent->GetActivePaper();
	if (pap) return(pap->GetFunctionFocus());
  return(NULL);
}




Paper *ApplicationControl::GetActivePaper()
{
  if (parent) return(parent->GetActivePaper());
	return(NULL);
}



void ApplicationControl::RecFunctionStatus()
{
	class m_function_object *func=GetActiveFunction();
  class string_object *string;
	if (func) {
		expmodecombo->setCurrentItem(func->GetDigitMode());
		digitcount->setCurrentItem((func->GetDigits()/2)-1);
    string=func->GetFocus();
    if (string) fontsizecombo->setCurrentItem(string->GetFontSize());
    answercombo->setCurrentItem(func->GetAnswerMode());
    rational->setOn(func->GetRationalMode());
	}	
}




void ApplicationControl::SendFunctionStatus()
{
	class m_function_object *func=GetActiveFunction();
	if (func) {
    func->SetModify();
		func->SetDigitMode(expmodecombo->currentItem());
		func->SetDigits(2+(digitcount->currentItem()*2));
    func->SetAnswerMode(answercombo->currentItem());
    func->SetRationalMode(rational->isOn());
	}	
}




void ApplicationControl::ZoomChanged(int x)
{
	if (parent) {
		Paper *pap=GetActivePaper();
		if (pap) pap->UpDate();
	}
}



int ApplicationControl::ZoomFactor()
{
  return(zoomcombo->currentText().toInt());
}




void ApplicationControl::ZoomFactor(int z)
{
  int i,index;
  int count=zoomcombo->count();	
  for(i=0;i<count;i++) if (zoomcombo->text(i).toInt()==z) { index=i; break; }
  zoomcombo->setCurrentItem(index);
}




void ApplicationControl::NextZoomFactor()
{
 int index=zoomcombo->currentItem();
 int count=zoomcombo->count();
 if ((index+1)<count) zoomcombo->setCurrentItem(index+1);
 ZoomChanged(0);
}




void ApplicationControl::PrevZoomFactor()
{
 int index=zoomcombo->currentItem();
 if (index>0) zoomcombo->setCurrentItem(index-1);
 ZoomChanged(0);
}




void ApplicationControl::AddError(QString er)
{
parent->statusBar()->message(er);
}




QColor ApplicationControl::GetColor(int id)
{
return(color[id]);
}



QFont ApplicationControl::GetFont(int id)
{
if (id==FONT_TYPE_SYMBOL) ErrorReport("App::GetFont","Illegal request");
return(font[id]);
}



int ApplicationControl::GetFontSize(int id)
{
return(fontsize[id]);
}




#include "Tools.cpp"
