/*
 * MathPlanner 3.1.0 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */

#ifndef _MATHPLANNER_MATH_H
#define _MATHPLANNER_MATH_H


#include "Unit.h"
#include <math.h>

#define MN_NULL                 0
#define MN_REAL                 1
#define MN_COMPLEX              2
#define MN_VECTOR               4
#define MN_RATIONAL             8
#define MN_MATRIX               16

#define NEPER M_E
#define PI M_PI

struct mpl_complex {
        double r;
        double i;
};

struct mpl_rational {
        double a;
        double b;
};

struct mpl_rational_info {
        char pii_location; //0=No Pii, 1=Up 2=Lo
        bool root;
};


struct mpl_vector {
        double i;
        double j;
        double k;
};


/*
class mpl_matrix {
        double *values;
        int w,h;
        public:
        mpl_matrix(int x,int y);
        ~mpl_matrix();
};
*/

class math_node  {

        public:
        double R,I,J,K;

        //class mpl_matrix *matrix;

        int count; // count of sub nodes
        class math_node *sub;

        char pwrs[COUNT];

        int  type;

        math_node();
        math_node(double);
        math_node(mpl_rational);
        math_node(mpl_complex);
        math_node(mpl_vector);

        //math_node(mpl_matrix);

        ~math_node();

        void Clear();
        bool isNull();
				
        void SetReal(double);
				void SetComplex(mpl_complex);
				void SetRational(mpl_rational);

        int  Count();
        bool SetCount(int count);

				void SetNode(int no,class math_node);
        class math_node Node(int no);

        double Real();
        mpl_rational Rational();
        mpl_complex Complex();
 				mpl_vector Vector();

        //mpl_matrix *Matrix();
};

// TEXT FUNCTIONS

QString mpl_real2text_frac(double real,int digits,bool mode);
QString mpl_real2text_exp(double real,bool mode);
QString mpl_real2text_dec(double real,int digits);

// NODE FUNCTIONS

math_node mpl_add(math_node a,math_node b);
math_node mpl_sub(math_node a,math_node b);
math_node mpl_mul(math_node a,math_node b);
math_node mpl_div(math_node a,math_node b);
math_node mpl_neg(math_node a);

// VECTOR MATH

mpl_vector mpl_cross(mpl_vector a,mpl_vector b);
mpl_vector mpl_neg(mpl_vector a);
double mpl_length(mpl_vector a);
mpl_vector mpl_unitvector(mpl_vector a);

// REAL MATH

double mpl_factorial(double val);
double mpl_rad(double);
double mpl_deg(double);
double mpl_coth(double x);
double mpl_asinh(double x);
double mpl_acosh(double x);
double mpl_atanh(double x);
double mpl_acoth(double x);
bool mpl_modul(double a,double b);
double mpl_sin(double x);
double mpl_cos(double x);

// COMPLEX MATH

mpl_complex mpl_real2complex(double r);
mpl_complex mpl_opz(mpl_complex z);   // one per z
mpl_complex mpl_cnxroot(mpl_complex z,double n,double x);
mpl_complex mpl_cnroot(mpl_complex z,mpl_complex n);
mpl_complex mpl_cnpower(mpl_complex z,mpl_complex n);
mpl_complex mpl_ce(mpl_complex z);
mpl_complex mpl_cexp(mpl_complex z);
mpl_complex mpl_cln(mpl_complex z);
mpl_complex mpl_clog(mpl_complex z);
mpl_complex mpl_cneg(mpl_complex z);
mpl_complex mpl_ckunjugant(mpl_complex z);
mpl_complex mpl_cang(double a);
double mpl_cmodul(mpl_complex z);
double mpl_cargz(mpl_complex z);
double mpl_cImz(mpl_complex z);
double mpl_cRez(mpl_complex z);



// RATIONAL MATH

double mpl_rat2real(mpl_rational a);
mpl_rational mpl_power(mpl_rational a,double n);
mpl_rational mpl_decrade(mpl_rational r);

bool create_table_of_numbers(int n);
void delete_table_of_numbers();
bool create_root(double real,double *in,double *out);
bool real_to_rational(double real,mpl_rational *rat,mpl_rational_info *info=NULL,bool use_info=false,bool test=true);


#endif


