/*
 * MathPlanner 3.0.3 - Mathematical design tool.
 * Copyright(C) 2002  Jarmo Nikkanen
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation.
 *
 * You should have received a copy of the GNU General Public License with this program.
 *
 */

#include <stdio.h>
#include <math.h>

#include "AppControl.h"
#include "Paper.h"
#include "Error.h"
#include "Memory.h"
#include "ServerHeaders.h"
#include "Math.h"
#include "DragDrop.h"
#include "ConfigReader.h"
#include "Datastore.h"
#include "Function_object.h"
#include "Extern.h"
#include "Object_string.h"

#include <qclipboard.h>
#include <qapplication.h>
#include <qpaintdevicemetrics.h>
#include <qfont.h>
#include <qfileinfo.h>



DataStorage *Paper::BuildStorage() // Delete data returned by this function
{
  int i,c;
  DataStorage *ar,*msg;
  char name[30];
  msg=new DataStorage();
  MathFrame *f;

  if (function) {
      for (i=0,c=0;i<Function_count;i++) if (function[i]) {
      FunctionPosition[c]=FunctionPosition[i];
      f=function[i]; function[i]=NULL; function[c]=f;
      c++;
      }
  }

  msg->AddInt("Paper:FC",c);
  msg->AddData("Paper:FPos",(void *)FunctionPosition,c*sizeof(QPoint));

  if (function) {
  for (i=0,c=0;i<Function_count;i++) {
        if (function[i]) {
        sprintf(name,"Paper:FE%d",(int)c);
        c++;
        ar=function[i]->BuildStorage();
        if (ar) msg->AddDataStorage(name,ar,false);
        if (ar) delete ar;
        }
  }
  }
  Bug("string_object archived");

  return(msg);

}


void Paper::New()
{

 Prefs->Load();

 //AppControl->parent->setCaption(AppControl->parent->ApplicationCaption);   // Why Here ???

 if (AppControl->function_server) AppControl->function_server->DeleteAllFunctions();
 if (AppControl->integer_server) AppControl->integer_server->DeleteAllIntegers();

 int i;

 Focused_function=(-1);

 if (function && FunctionPosition) {
 for(i=0;i<Function_count;i++) {
        if (function[i]) {
                 delete function[i];
                 function[i]=NULL;
                 FunctionPosition[i]=QPoint(0,0);
        }
 }
 }

 UpDate();
}



void Paper::Record(DataStorage *msg)
{
  int i;
  int bytes;
  QPoint *DP;
  DataStorage *ar;
  char name[30];
  Bug("***********  START Paper::Record  *************");

  Function_count=msg->ReadInt("Paper:FC");

  if (Function_count<32) Function_count=32;

  SetFunctionSpace(Function_count);

  if (msg->Contains("Paper:FPos")) {
       msg->ReadData("Paper:FPos",(void **)&DP,&bytes);
  }

  int count=bytes/sizeof(QPoint);
  if (count>Function_count) ErrorReport("Paper:Record","Too many data nodes");
  for(i=0;i<count;i++) FunctionPosition[i]=DP[i];
  

  Bug("Start UnArchiving Function");

  if (function) {
  for (i=0;i<Function_count;i++) {
        sprintf(name,"Paper:FE%d",(int)i);
        if (msg->Contains(name)) {
           ar=new DataStorage();
           msg->ReadDataStorage(name,ar);
           int Ob=ar->ReadInt("Object");
           if (Ob==PAPER_OBJECT_FUNCTION) function[i]=new m_function_object(ar,AppControl,this);
           if (Ob==PAPER_OBJECT_IMAGE) function[i]=new PaperImage(ar,AppControl,this);
           if (Ob==PAPER_OBJECT_QPICTURE) function[i]=new PaperQPicture(ar,AppControl,this);
           if (Ob==PAPER_OBJECT_TEXT) function[i]=new PaperText(ar,AppControl,this);

       //  if (Ob==PAPER_OBJECT_GRAPH) function[i]=new PaperGraph(ar,AppControl,this);

           delete ar;
        }
  }
  }

  UpDate();
  Bug("Paper_object unarchived");
}



void Paper::CalculateAll()
{
int i,x,count;
class MathFrame *f;
Bug("START CALCULATE ALL");

AppControl->AddError("");

if (AppControl->integer_server) AppControl->integer_server->DeleteAllIntegers();
if (AppControl->function_server) AppControl->function_server->DeleteAllFunctions();

count=OrderFunctions();

x=0;
i=0;
for(x=0;x<count;x++) {
   i=Order[x];
   if (i!=(-1)) {
     f=function[i];
     if (f) {

       // **************  MATH FRAME EXECUTION  ******************

       if (f->ObjectType==PAPER_OBJECT_FUNCTION) {
           m_function_object *fo=(m_function_object *)f;

           fo->SetModify();

           fo->Build();
           AppControl->integer_server->SetLocalValues(fo);

           fo->Calculate();

       }

       /*
       if (f->ObjectType==PAPER_OBJECT_GRAPH) {
           PaperGraph *fo=(PaperGraph *)f;
           fo->DrawFunctions();
       }  */



       // **************  MATH FRAME EXECUTION  ******************

     }
   }  
}

Bug("calculate completed");
}



int Paper::OrderFunctions()
{
 int i,k,top_f,last,count;
 int pos,x,y,top,line,max;

 max=2147483640;
 line=0;
 count=0;
 last=0;

 for(i=0;i<Function_count;i++) {
    if (function[i]) last=i,count++;
 }
 
 for(i=0;i<=(last+1);i++) {
    top=max;
    top_f=(-1);
    for(k=0;k<=last;k++) {
       if (function[k]) {
       y=FunctionPosition[k].y();
       x=FunctionPosition[k].x();
       pos=(y*2000)+(x/5);
       if (pos<top && pos>line) top=pos,top_f=k;
       }
    }
    if (top!=max) {
       Order[i]=top_f;
       line=top;
    }
    else {
       Order[i]=(-1);
       break; //i=last+10;
    }   
 }
 return(count);   
}



void Paper::UpDate()
{
     class m_function_object *fuo;
     class MathFrame *f;
     int i;

     Bug("******** UPDATE CALLED ************");

     PaperLimits=Prefs->Rect("PaperWidth","PaperHeight");
     Grid=Prefs->Int("GridSize");
     GridColor=Prefs->Color("GridColor");

     Printable=QRect(Prefs->Int("LeftMargin"),Prefs->Int("TopMargin"),
         PaperLimits.width()-Prefs->Int("RightMargin")-Prefs->Int("LeftMargin"),
         PaperLimits.height()-Prefs->Int("BottomMargin")-Prefs->Int("TopMargin"));

     int x=PaperLimits.width();
     int y=PaperLimits.height();
     int z=AppControl->ZoomFactor();

     QRect re=Scale(z,QRect(0,0,x,y));

     if (geometry().size()!=re.size()) setGeometry(re);

     setBackgroundColor(Prefs->Color("PaperColor"));



     CalculateAll();



     for(i=0;i<Function_count;i++) {
     f=function[i];
     if (f) {
       if (f->ObjectType==PAPER_OBJECT_FUNCTION) {
           fuo=(m_function_object *)f;
           fuo->SendCall(CALL_SYNTAXCHK);
           Bug("******** UPDATE: CALCULATE BOUNDS ************");
           fuo->CalculateBounds();
           Bug("******** UPDATE: MOVETO ************");
           fuo->MoveTo(FixedFunctionPosition(fuo));
           Bug("Passed");
       }
     }
     }


   if (AppControl->function_server) {
     AppControl->CreateFunctionMenu(AppControl->function_server->CreateFunctionList());
   }
   Bug("OK");

   update();
}


void Paper::CommandToString(int com,int dat)
{
	string_object *str;
	m_function_object *func=GetFunctionFocus();
	if (func) {
		str=func->GetFocus();
		if (str) str->MessageReceived(com,dat);
	}
}




void Paper::FocusNone()
{
	Focused_function=(-1);
	UpDate();
}


// FOCUS CONTROL    FOCUS CONTROL     FOCUS CONTROL


void Paper::UnFocusFunction(int no)
{
	if (Focused_function==no) {
		Focused_function=(-1);
		if (no!=(-1)) DrawFunction(function[no]);
	}
}


void Paper::UnFocusCurrentFunction()
{
        CalculateAll();

	      int no=Focused_function;
        Focused_function=(-1);
        if (no!=(-1)) {
              if (function[no]) {

                 if (function[no]->ObjectType==PAPER_OBJECT_FUNCTION) {
           					m_function_object *fo=(m_function_object *)function[no];
                    fo->SendCall(CALL_SYNTAXCHK);
           					fo->SetModify();
                    fo->AutoDelete();
        				 }

                 DrawFunction(function[no]);

              }
        }
}


void Paper::FocusFunction(int no)
{
        Focused_function=no;
        if (no!=(-1)) DrawFunction(function[no]);
        AppControl->Read();
}


MathFrame *Paper::GetFocus()
{
	if (Focused_function!=(-1)) return(function[Focused_function]);
	return(NULL);
}


m_function_object *Paper::GetFunctionFocus()
{
	if (Focused_function!=(-1)) {
	        if (function[Focused_function]->ObjectType==PAPER_OBJECT_FUNCTION)
	        	return((m_function_object *)function[Focused_function]);
	}
	return(NULL);
}

        	
// **********************************************************




int Paper::FindFunctionIndex(class MathFrame *f)
{
int i;
for(i=0;i<Function_count;i++) if (function[i]==f) return(i);
return(-1);
}


void Paper::DeleteEmptyFunctions()
{
int i;
for(i=0;i<Function_count;i++) if (function[i]) {
            if (function[i]->IsEmpty()) DeleteFunction(i);
}
}


void Paper::DrawGrid(QPainter *paint)
{

     int i;
     int x=Grid;
     int y=Grid;

     QColor c=GridColor;
     QColor cc(c.red()*8/10,c.green()*8/10,c.blue()*8/10);


     QRect rect=Printable;

     int ri=rect.right();
     int bo=rect.bottom();
     int le=rect.left();
     int to=rect.top();

     QPen pen(c,0,Qt::DashLine);
     QPen pen2(cc,0,Qt::DashLine);

     paint->setPen(pen);
     paint->setWindow(PaperLimits);
     for(i=x+rect.left();i<ri;i+=x) paint->drawLine(QPoint(i,to),QPoint(i,bo));
     for(i=y+rect.top();i<bo;i+=y) paint->drawLine(QPoint(le,i),QPoint(ri,i));
     paint->setPen(pen2);
     paint->drawRect(rect);

}




void Paper::DrawFunction(MathFrame *f)
{
        int zoom=AppControl->ZoomFactor();

        int i=FindFunctionIndex(f);

        if (i!=(-1)) {
        Bug("*** DrawFunction Called ***");

        if (f->ObjectType==PAPER_OBJECT_FUNCTION) {
           	m_function_object *fo=(m_function_object *)f;
           	fo->Calculate();
        }

        f->CalculateBounds();
        f->MoveTo(FixedFunctionPosition(f));

        QRect bo=f->Bounds()|f->OldBounds();

        int x=(bo.left()*zoom/100)-2;
        int y=(bo.top()*zoom/100)-2;
        int xx=(bo.right()*zoom/100)+2;
        int yy=(bo.bottom()*zoom/100)+2;

        QRect r;
        r.setCoords(x,y,xx,yy);

        repaint(x,y,xx-x,yy-y,false);

        }
}



void Paper::Snap(QPoint p,int *x,int *y)
{
  int sx=Grid;
  int sy=Grid;
  *x=((p.x()-(sx/2))/sx)*sx;
  *y=((p.y()-(sy/2))/sy)*sy;
}



QPoint Paper::ScreenToDocument(QPoint p)
{ 	
  int zoom=AppControl->ZoomFactor(); 	
	int x=p.x()*100/zoom;
	int y=p.y()*100/zoom;	
	return(QPoint(x,y));
}


QRect Paper::ScreenToDocument(QRect r)
{
        int zoom=AppControl->ZoomFactor();
        int x=r.left()*100/zoom;
        int y=r.top()*100/zoom;	
	      int xx=r.right()*100/zoom;
	      int yy=r.bottom()*100/zoom;	
        QRect re;
        re.setCoords(x,y,xx,yy);
        return(re);
}



void Paper::paintEvent(QPaintEvent *pe)
{

  QPixmap buffer(size());

  QPainter paint;
  paint.begin(&buffer,this);

  paint.setClipRect(pe->rect());
  paint.setClipping(true);
  paint.eraseRect(pe->rect());

  if (Grid) DrawGrid(&paint);

  paint.setWindow(Scale(100,PaperLimits));
  paint.setViewport(0,0,width(),height());

  Draw(&paint,pe->rect(),false);

  paint.end();

  bitBlt(this,pe->rect().topLeft(),&buffer,pe->rect());
}


void Paper::Draw(QPainter *pd,QRect drawrect,bool all)
{
  MathFrame *f;
  int i;
  for(i=0;i<Function_count;i++) {
       if (function[i]) {

       		f=function[i];

       		if (ScreenToDocument(drawrect).intersects(f->Bounds()) || all) {
       		      Bug("**** DRAWING FUNCTION *****");   		
                      f->Draw(pd);
                      pd->flush();	
       		}
       }
  }
}

QImage Paper::CreateImage(MathFrame *function)
{
        QPixmap pixmap(function->bounds.size());
        function->MoveTo(QPoint(0,0));
        QPainter paint;
        paint.begin(&pixmap);
        function->Draw(&paint);
        paint.end();
        QImage image=pixmap.convertToImage();
	      return(image);
}


void Paper::AddGraph()
{	
   /*
        MathFrame *f=GetFocus();
        FocusNone();
        if (f)
        if (f->ObjectType==PAPER_OBJECT_GRAPH) {
             AppControl->Graph->SetGraph((class PaperGraph *)f,false);
             return;
        }

        class PaperGraph *pg=new PaperGraph(QRect(0,0,200,150),AppControl,this);
	AddFunction(QPoint(0,0),pg);
	AppControl->Graph->SetGraph(pg,true);	
*/

}


void Paper::DragObject()
{

   int Type=-1;
   MathFrame *f=GetFocus();

   if (f) {
   Type=f->ObjectType;

   if (Type==PAPER_OBJECT_FUNCTION) QApplication::clipboard()->setData(new MathDragObject(this,f));

   if (Type==PAPER_OBJECT_IMAGE) {
                  PaperImage *I=(PaperImage *)f;
                  QApplication::clipboard()->setData(new ImageDragObject(I->Pixmap()));
   }
   if (Type==PAPER_OBJECT_QPICTURE) {
                  PaperQPicture *I=(PaperQPicture *)f;
                  QApplication::clipboard()->setData(new QPictureDragObject(I->Picture()));
   }
   if (Type==PAPER_OBJECT_TEXT) {
                  PaperText *I=(PaperText *)f;
                  QApplication::clipboard()->setData(new QTextDrag(I->Text()));
   }

   /*
   if (Type==PAPER_OBJECT_GRAPH) QApplication::clipboard()->setData(new MathDragObject(this,f));
   */

   }

}



void Paper::DropObject()
{

  QImage img;
	QMimeSource *mime=NULL;
	int ret;
  ret=0;
	if (ret==0) {
		
	QClipboard *cp=QApplication::clipboard();
	
  if (cp) mime=cp->data();
  else QMessageBox::information( this, "MathPlanner","Critical.. No Clipboard");
     	 	
  if (mime) {

  	
 	if (QTextDrag::canDecode(mime)) {
 	  QString text;
		if (QTextDrag::decode(mime,text)) {
			AddFunction(QPoint(0,0),new PaperText(text,AppControl,this));
			return;
		}
	}
 	
 	 	
     	

  if (MathDragObject::canDecode(mime)) {   	 	
     	DataStorage *d=MathDragObject::Decode(mime);
        if (d) {
        	MathFrame *f;
        	//PaperGraph *g;
                int Type=d->ReadInt("Object");
                if (Type==PAPER_OBJECT_FUNCTION) f=new m_function_object(d,AppControl,this);
             /*   if (Type==PAPER_OBJECT_GRAPH) {
                              g=new PaperGraph(d,AppControl,this);
                              g->DrawFunctions();
                              f=g;
                }*/
         	delete d;
          AddFunction(QPoint(0,0),f);
          return;
        }
  }


  if (QPictureDragObject::canDecode(mime)) {
		QPicture *pic=QPictureDragObject::Decode(mime);
		AddFunction(QPoint(0,0),new PaperQPicture(pic,AppControl,this));
		return;
	}


   if (ImageDragObject::canDecode(mime)) {
		img=ImageDragObject::Decode(mime);
		AddFunction(QPoint(0,0),new PaperImage(img,AppControl,this));
		return;
	}
	
	
 	} // if mime
 	} // message



}

     	
     	  	

void Paper::dragEnterEvent(QDragEnterEvent *event)
{
Bug("%%%%%%%%%%%%%% dragEnterEvent");

if (MathDragObject::canDecode(event)) {
           Bug("%% Accept");
           event->accept(true);
           return;
}

if (QTextDrag::canDecode(event)) {
           Bug("%% Accept");
           event->accept(true);
           return;
}

if (QPictureDragObject::canDecode(event)) {
	         Bug("%% Accept");
           event->accept(true);
           return;
}	

if (ImageDragObject::canDecode(event)) {
           Bug("%% Accept");
           event->accept(true);
           return;
}

Bug("%% End");

}



void Paper::dropEvent(QDropEvent* event)
{

int x,y;
QImage img;
QString text,no;
const char *tp;
int i;


if (AppControl->Prefs->Int("Show_drop_formats")==1) {
  tp=event->format(0);
	i=1;
	while(tp) {
		text.append(tp);
		text.append("\n");
		tp=event->format(i++);
	}	
	QMessageBox::information( this, "QMimeSource::format() Output",QString(text+"\n:are supported"));
}


QPoint p=ScreenToDocument(event->pos());

Snap(p,&x,&y);

Bug("%%%%%%%%% dropEvent");

if (MathDragObject::canDecode(event)) {
	DataStorage *d=MathDragObject::Decode(event);
	if (d) {
	        MathFrame *f;
	        //PaperGraph *g;
                int Type=d->ReadInt("Object");
                if (Type==PAPER_OBJECT_FUNCTION) f=new m_function_object(d,AppControl,this);
            /*    if (Type==PAPER_OBJECT_GRAPH) {
                	g=new PaperGraph(d,AppControl,this);
                	g->DrawFunctions();
                	f=g;
                }*/
		delete d;
		AddFunction(QPoint(x,y),f);
    UpDate();
		return;
        }
}

if (QTextDrag::canDecode(event)) {
 	
		if (QTextDrag::decode(event,text)) {

      if (text.find("file:")==0) {

      QString file_name=text;
      file_name.remove(0,5);
      QFileInfo info(file_name);
      int type=0;
      QString ext=info.extension();
      ext.lower();

      if (ext==QString("txt")) type=1;
      if (ext==QString("html")) type=1;
      if (ext==QString("htm")) type=1;
      if (ext==QString("xpm")) type=2;
      if (ext==QString("png")) type=2;

      if (type==1) {
         QFile file(file_name);
         if ( file.open(IO_ReadOnly) ) {    // file opened successfully
            QTextStream t( &file );
            AddFunction(QPoint(x,y),new PaperText(t.read(),AppControl,this));
         }
      }

      if (type==2) {
         QImage img(file_name);
         AddFunction(QPoint(x,y),new PaperImage(img,AppControl,this));
      }
      }

      else AddFunction(QPoint(x,y),new PaperText(text,AppControl,this));

			return;
		}
		return;
}


if (QPictureDragObject::canDecode(event)) {
	QPicture *pic=QPictureDragObject::Decode(event);
	AddFunction(QPoint(x,y),new PaperQPicture(pic,AppControl,this));
	return;
}


if (ImageDragObject::canDecode(event)) {
	img=ImageDragObject::Decode(event);
	AddFunction(QPoint(x,y),new PaperImage(img,AppControl,this));
	return;	
}



	tp=event->format(0);
	i=1;
	while(tp) {
		text.append(tp);
		text.append("\n");
		tp=event->format(i++);
	}
	
	QMessageBox::information( this, "QMimeSource::format() Output",
 	QString("UnAble to drop this object..\ntype is not supported\n\n")+text+"\n:are supported");



}




void Paper::mouseReleaseEvent(QMouseEvent *me)
{
Begin_move=false;
AppControl->Timer.stop();
}




void Paper::mouseMoveEvent(QMouseEvent *me)
{
  int x,y;
  bool out;

  AppControl->Timer.start(1000,true);

  if (Focused_function!=(-1) && function && (Begin_move==true || Begin_resize==true)) {

  MathFrame *f=GetFocus();

  QPoint point=me->pos();
  QPoint p=ScreenToDocument(point)-Begin_move_align;

  Snap(p,&x,&y);
  point=QPoint(x,y);

  // Check the print rect

  QRect print=Scale(100,PaperLimits);

  if (Begin_move==true) {  // DO NOT LET IT PASS OUTSIDE

  QPoint po=point;

  out=false;
  if (po.x()<print.left()) out=true;
  if (po.y()<print.top()) out=true;
  if ((po.x()+f->Bounds().width())>print.right()) out=true;
  if ((po.y()+f->Bounds().height())>print.bottom()) out=true;

  QPoint pp=FixedFunctionPosition(f);  // IF IT ALLREADY LIES OUTSIDE -> LET IT BE MOVED

  if (pp.x()<print.left()) out=false;
  if (pp.y()<print.top()) out=false;
  if ((pp.x()+f->Bounds().width())>print.right()) out=false;
  if ((pp.y()+f->Bounds().height())>print.bottom()) out=false;

  if (out==false) {
  	if (FunctionPosition[Focused_function]!=point) {
      FunctionPosition[Focused_function]=point;
  		DrawFunction(f);
		}
  }
  }



  if (Begin_resize==true) {

  out=false;
  if ((point.x()-10)<f->Bounds().left()) out=true;
  if ((point.y()-10)<f->Bounds().top()) out=true;
  if (point.x()>print.right()) out=true;
  if (point.y()>print.bottom()) out=true;

  if (out==false) {
  QRect bo=f->Bounds();
  bo.setRight(point.x());
  bo.setBottom(point.y());
  f->SetBounds(bo);
  DrawFunction(f);
  }
  }


  }
}


void Paper::StartDragging()
{

      MathFrame *f=GetFocus();
      AppControl->Timer.stop();
      if (f) {
      int Type=f->ObjectType;


      if (Type==PAPER_OBJECT_FUNCTION) {
      		QDragObject *mdrag=new MathDragObject(this,f);
      		mdrag->setPixmap(AppControl->DragImage,QPoint(16,16));
      		Begin_move=false;
      		mdrag->drag();
      }
      /*
      if (Type==PAPER_OBJECT_GRAPH) {
      		QDragObject *mdrag=new MathDragObject(this,f);
      		mdrag->setPixmap(AppControl->DragImage,QPoint(16,16));
      		Begin_move=false;
      		mdrag->drag();
      } */


      if (Type==PAPER_OBJECT_IMAGE) {
        	PaperImage *I=(PaperImage *)f;
      		QDragObject *mdrag=new ImageDragObject(I->Pixmap(),this);
      		mdrag->setPixmap(AppControl->DragImage,QPoint(16,16));
      		Begin_move=false;
      		mdrag->drag();
      }


      if (Type==PAPER_OBJECT_QPICTURE) {
      		PaperQPicture *I=(PaperQPicture *)f;
                QDragObject *mdrag=new QPictureDragObject(I->Picture(),this);
                mdrag->setPixmap(AppControl->DragImage,QPoint(16,16));
      		Begin_move=false;
      		mdrag->drag();
      }

       if (Type==PAPER_OBJECT_TEXT) {
      		PaperText *I=(PaperText *)f;
                QDragObject *mdrag=new QTextDrag(I->Text(),this);
                mdrag->setPixmap(AppControl->DragImage,QPoint(16,16));
      		Begin_move=false;
      		mdrag->drag();
      }

      }

}


QPoint Paper::FixedFunctionPosition(MathFrame *f,bool pm)
{
        if (f) {
        int i=FindFunctionIndex(f);
        if (i!=(-1)) {
	      QPoint p=FunctionPosition[i];
 	      return(p);
 	
 	}}
 	return(QPoint(0,0));
}

  		

void Paper::mousePressEvent(QMouseEvent *me)
{
  int x,y,zoom,i;
  bool newfunction;

  AppControl->Timer.start(1000,true);

  QPoint point=me->pos();

  DeleteEmptyFunctions();

  Bug("Here");
  zoom=AppControl->ZoomFactor();

  QPoint p=ScreenToDocument(point);
  Snap(p,&x,&y);
  point=QPoint(x,y);

  newfunction=true;

  for(i=0;i<Function_count;i++) {
  	if (function[i]) {	
  		if (function[i]->IsInBounds(p)) {
  		
  		  Begin_resize=Begin_move=false;
  		  newfunction=false;

  			if (function[i]->IsInResizeArea(p)) {
  				Begin_resize=true;
  				Begin_move_align=point-function[i]->Bounds().bottomRight();
  			}
  			else {
  				Begin_move=true;
  				Begin_move_align=point-FunctionPosition[i]; //FixedFunctionPosition(function[i],true);
  			}

  			if (Focused_function!=i) UnFocusCurrentFunction();

  			FocusFunction(i);			
  			function[i]->MousePressed(p);		
  		}
  	}
  }


  if (newfunction) {

    UnFocusCurrentFunction();

  	if (CheckPoint(point) && me->button()==Qt::LeftButton) {
            AddFunction(point,new m_function_object(p,AppControl,this));
  	}
  }

}



bool Paper::CheckPoint(QPoint p)
{
int i;
for(i=0;i<Function_count;i++) if (FunctionPosition[i]==p) return(false);
return(true);
}



void Paper::DeleteCurrentFunction()
{
	int no=Focused_function;
	if (no!=(-1)) DeleteFunction(no);
}
	

void Paper::DeleteFunction(int no)
{
if (no>=0) {
if (function[no]) {
       QRect bo=function[no]->Bounds();
       UnFocusFunction(no);
       Bug("Delete Function");
       Bug((long int)no);
       delete function[no];
       function[no]=NULL;
       FunctionPosition[no]=QPoint(0,0);
       update(bo);
}
}
}


void Paper::SetNewFunctionAfter(MathFrame *func)
{
int x,y;
int F=FindFunctionIndex(func);
QPoint p=FunctionPosition[F];
int zoom=AppControl->ZoomFactor();
x=p.x();
y=p.y();

y+=(func->Bounds().height()*100/zoom);
y+=Grid;

Snap(QPoint(x,y),&x,&y);

AddFunction(QPoint(x,y),new m_function_object(QPoint(x*zoom,y*zoom),AppControl,this));
}




void Paper::AddFunction(QPoint p,MathFrame *func,bool flag)
{
int i;

AppControl->CreateControlMenu(NULL);  // Reset control menu

for(i=0;i<Function_count;i++) {
         if (function[i]==NULL) {
                Bug("Paper:AddFunction Adding function");
                function[i]=func;

                func->CalculateBounds();

                FunctionPosition[i]=p;

                func->MoveTo(FixedFunctionPosition(func));

                Bug("Paper:AddFunction Function Added");
                FocusFunction(i);
                return;
         }
}

if (flag==false) {  // Prevent Looping
	AddFunctionSpace(50);
	AddFunction(p,func,true);
	return;
}

ErrorReport("Paper::AddFunction()","No space in function list");
}



QRect Paper::Scale(int zoom,QRect b)
{
	QPaintDeviceMetrics met(this);	
	int w=b.width()*met.logicalDpiX()*zoom/2500;
	int h=b.height()*met.logicalDpiY()*zoom/2500;
	return(QRect(0,0,w,h));
}



Paper::Paper(QWidget *pa,QRect bounds,class ApplicationControl *Aplc)
:QWidget(pa)
{
  setAcceptDrops(true);
  Prefs=Aplc->Prefs;
  AppControl=Aplc;

  setFocusPolicy(QWidget::ClickFocus);
  setGeometry(Scale(100,bounds));

  setBackgroundColor(Prefs->Color("PaperColor"));
  PaperLimits=Prefs->Rect("PaperWidth","PaperHeight");
  Grid=Prefs->Int("GridSize");
  GridColor=Prefs->Color("GridColor");

  //exclude_i=(bool)Prefs->Int("MapItoImag");

  Printable=QRect(Prefs->Int("LeftMargin"),Prefs->Int("TopMargin"),
     PaperLimits.width()-Prefs->Int("RightMargin")-Prefs->Int("LeftMargin"),
       PaperLimits.height()-Prefs->Int("BottomMargin")-Prefs->Int("TopMargin"));


  function_mem=new NMemory();
  Order_mem=new NMemory();
  Position_mem=new NMemory();

  Focused_function=-1;

  SetFunctionSpace(20);

  AppControl->ZoomFactor(100);

}


void Paper::AddFunctionSpace(int amount)
{
	Function_count+=amount;
	function_mem->SetSizeTo(Function_count*sizeof(MathFrame *),true);
 	Order_mem->SetSizeTo(Function_count*sizeof(int),true);
 	Position_mem->SetSizeTo(Function_count*sizeof(QPoint),true);
	function=(MathFrame **)function_mem->Address();
  Order=(int *)Order_mem->Address();
  FunctionPosition=(QPoint *)Position_mem->Address();
}


void Paper::SetFunctionSpace(int amount)
{
	Function_count=amount;
	
	function_mem->SetSizeTo(Function_count*sizeof(class MathFrame *),false);
 	Order_mem->SetSizeTo(Function_count*sizeof(int),false);
 	Position_mem->SetSizeTo(Function_count*sizeof(QPoint),false);
 	
	function=(class MathFrame **)function_mem->Address();
  Order=(int *)Order_mem->Address();
  FunctionPosition=(QPoint *)Position_mem->Address();
}
	
	



void  Paper::keyPressEvent(QKeyEvent *ke)
{
  //Pass Key Codes To Object

  int ch;
  int co,code;
  key_code key;
  int i;
  QString uni;
  Bug("&&& KeyDown");

  char std_operator_keys[10]={"+-*/=:<>"};
  std_operator_keys[8]=-1;

  char non_integers[40]={"+-*!/=()[]{}#$%&?^,;:<>|"};
  non_integers[24]=-1;

  // exclude edit keys
  int specialkeys[40]={Qt::Key_Insert,Qt::Key_Home,Qt::Key_PageUp,Qt::Key_PageDown,
                    Qt::Key_Tab,Qt::Key_Enter,Qt::Key_End,Qt::Key_Space,
                    Qt::Key_Up,Qt::Key_Down,Qt::Key_Return,-1};


  int editkeys[40]={Qt::Key_Left,Qt::Key_Right,Qt::Key_Delete,Qt::Key_Backspace,-1};

  key.code=0;
  key.unicode=QString("");
  key.number=false;
  key.spec=false;
  key.edit=false,
  key.non=false;
  key.oper=false;
  key.spec_num=false;
  key.integer=false;
  key.insert_mode=MODE_AUTO; // MODE_ADD_RIGHT

       m_function_object *func_o;    	
       func_o=GetFunctionFocus();

       if (func_o) {

       ch=key.ascii=ke->ascii();
       uni=key.unicode=ke->text();
       code=key.code=ke->key();
       key.modifiers=ke->state();




       // KEY TYPES

       if (ch>='0' && ch<='9') key.number=true;   // Number Key

       if (ch>='a' && ch<='z') key.integer=true;   // Integer Key
       if (ch>='A' && ch<='Z') key.integer=true;
       if (uni.at(0)>=QChar(0x390) && uni.at(0)<=QChar(0x3D0)) key.integer=true;

       //if (exclude_i && ch=='i') key.integer=false;
       //Add: key type text

       i=co=0;
       while (co!=(-1)) {
         co=editkeys[i++];
         if (code==co) key.edit=true,co=(-1);  // Edit Key
       }

        i=co=0;
       while (co!=(-1)) {
         co=specialkeys[i++];
         if (code==co) key.spec=true,co=(-1);  // Special Key
       }


       i=co=0;
       while (co!=(-1)) {
         co=non_integers[i++];
         if (ch==co) key.non=true,co=(-1);  // Non integer Key
       }

       i=co=0;
       while (co!=(-1)) {
         co=std_operator_keys[i++];
         if (ch==co) key.oper=true,co=(-1);  // Non integer Key
       }

       if (ch=='.') key.spec_num=true;

       bool pass=key.number|key.edit|key.integer|key.non|key.spec|key.spec_num|key.oper;


       if (code==Qt::Key_Return) CalculateAll();


       if (ke->state()!=Qt::ControlButton && code!=Qt::Key_Control && pass) {
               func_o->KeyReceived(key);
       }

       }

}








